@@ -28,6 +28,7 @@ import threading
 from logging.handlers import RotatingFileHandler
 from os.path import expanduser
 import platform
+import stdiomask
 
 from c8ylp.rest_client.c8yclient import CumulocityClient
 from c8ylp.tcp_socket.tcp_server import TCPServer
@@ -144,8 +145,8 @@ def start():
             help()
     if script_mode:
         reconnects = -1
-    validate_parameter(host, device, extype, config_name,
-                       tenant, user, password, token)
+    password = validate_parameter(host, device, extype, config_name,
+                       tenant, user, password, token, script_mode)
     if use_pid:
         upsert_pid_file(device, host, config_name, user)
     if kill_instances:
@@ -162,6 +163,13 @@ def start():
         client.validate_token()
     else:
         session = client.retrieve_token()
+    if session == None and tfacode == None:
+        if not script_mode:
+            tfacode = stdiomask.getpass(prompt='Enter your TFA-Token:  ', mask='*')
+            client.tfacode = tfacode
+            session = client.retrieve_token()
+    if session == None:
+        sys.exit(1)
     mor = client.read_mo(device, extype)
     config_id = client.get_config_id(mor, config_name)
     device_id = client.get_device_id(mor)
@@ -275,7 +283,7 @@ def kill_existing_instances():
             clean_pid_file(other_pid)
 
 
-def validate_parameter(host, device, extpye, config_name, tenant, user, password, token):
+def validate_parameter(host, device, extpye, config_name, tenant, user, password, token, script_mode):
     if not host:
         logging.error(f'Hostname is missing!')
         print('Mandatory parameter -h, --hostname is missing')
@@ -308,9 +316,12 @@ def validate_parameter(host, device, extpye, config_name, tenant, user, password
 
     if not password and not token:
         logging.error(f'Password is missing!')
-        print('Mandatory parameter -p, --password is missing')
-        print(_help_message())
-        sys.exit(1)
+        if not script_mode:
+            password = stdiomask.getpass(prompt='Enter your Cumulocity Password: ', mask='*')
+        #print('Mandatory parameter -p, --password is missing')
+        #print(_help_message())
+        #sys.exit(1)
+    return password
 
 
 def help():
@@ -337,7 +348,7 @@ def _help_message() -> str:
                ' --tcptimeout           OPTIONAL, Timeout in sec. for inactivity. Can be activited with values > 0. Default: Deactivated\n'
                ' -v, --verbose          OPTIONAL, Print Debug Information into the Logs and Console when set.\n'
                ' -s, --scriptmode       OPTIONAL, Stops the TCP Server after first connection. No automatical restart!\n'
-               ' --ignore-ssl-validate  OPTIONAL, Ignore Validation for SSL Certificates while connecting to Websocket'
+               ' --ignore-ssl-validate  OPTIONAL, Ignore Validation for SSL Certificates while connecting to Websocket\n'
                ' --use-pid              OPTIONAL, Will create a PID-File in /var/run/c8ylp to store all Processes currently running.\n'
                ' --reconnects           OPTIONAL, The number of reconnects to the Cloud Remote Service. 0 for infinite reconnects. Default: 5'
                '\n')
